@@ -1,4 +1,4 @@
-# Copyright 2013 Jeffrey Kegler
+# Copyright 2015 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -26,7 +26,7 @@ no warnings qw(recursion qw);
 use strict;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '2.073_001';
+$VERSION        = '2.103_008';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -70,8 +70,7 @@ sub Marpa::R2::Grammar::new {
 
     $grammar->[Marpa::R2::Internal::Grammar::SYMBOLS]            = [];
     $grammar->[Marpa::R2::Internal::Grammar::RULES]              = [];
-    $grammar->[Marpa::R2::Internal::Grammar::RULE_BY_NAME]       = {};
-    $grammar->[Marpa::R2::Internal::Grammar::RULE_NAME_REQUIRED] = 0;
+    $grammar->[Marpa::R2::Internal::Grammar::RULE_ID_BY_TAG]     = {};
 
     my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C] =
         Marpa::R2::Thin::G->new( { if => 1 } );
@@ -130,7 +129,6 @@ sub Marpa::R2::Grammar::set {
                 default_empty_action
                 default_rank
                 inaccessible_ok
-                rule_name_required
                 rules
                 source
                 start
@@ -187,11 +185,6 @@ sub Marpa::R2::Grammar::set {
             }
         } ## end if ( defined( my $value = $args->{'symbols'} ) )
 
-        if ( defined( my $value = $args->{'rule_name_required'} ) ) {
-            $grammar->[Marpa::R2::Internal::Grammar::RULE_NAME_REQUIRED] =
-                !!$value;
-        }
-
         if ( defined( my $value = $args->{'terminals'} ) ) {
             Marpa::R2::exception(
                 'terminals option not allowed after grammar is precomputed')
@@ -613,27 +606,245 @@ sub Marpa::R2::Grammar::precompute {
     } ## end if ( $grammar->[Marpa::R2::Internal::Grammar::WARNINGS...])
 
     # If we are using scannerless parsing, set that up
-    my $cc_hash = $grammar->[Marpa::R2::Internal::Grammar::CHARACTER_CLASSES];
-    if (defined $cc_hash) {
-        my $class_table = $grammar->[Marpa::R2::Internal::Grammar::CHARACTER_CLASS_TABLE] = [];
-        for my $cc_symbol ( sort keys %{$cc_hash} ) {
-            my $regex = $cc_hash->{$cc_symbol};
-            push @{$class_table},
-                [ $grammar->thin_symbol($cc_symbol), $regex ];
-        }
-    }
-
-    # Save some memory
-    $grammar->[Marpa::R2::Internal::Grammar::CHARACTER_CLASSES] = undef;
+    Marpa::R2::exception("Internal error; precompute called for SLIF grammar")
+        if $grammar->[Marpa::R2::Internal::Grammar::CHARACTER_CLASSES];
 
     return $grammar;
 
 } ## end sub Marpa::R2::Grammar::precompute
 
-sub Marpa::R2::Grammar::rule_by_name {
-    my ( $grammar, $name ) = @_;
-    return $grammar->[Marpa::R2::Internal::Grammar::RULE_BY_NAME]->{$name};
-}
+# A custom precompute for SLIF grammars
+sub Marpa::R2::Internal::Grammar::slif_precompute {
+    my $grammar = shift;
+
+    my $rules     = $grammar->[Marpa::R2::Internal::Grammar::RULES];
+    my $symbols     = $grammar->[Marpa::R2::Internal::Grammar::SYMBOLS];
+    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
+    my $trace_fh =
+        $grammar->[Marpa::R2::Internal::Grammar::TRACE_FILE_HANDLE];
+
+    my $problems = $grammar->[Marpa::R2::Internal::Grammar::PROBLEMS];
+    if ($problems) {
+        Marpa::R2::exception(
+            Marpa::R2::Grammar::show_problems($grammar),
+            "Second attempt to precompute grammar with fatal problems\n",
+            'Marpa::R2 cannot proceed'
+        );
+    } ## end if ($problems)
+
+    return if $grammar_c->is_precomputed();
+    if ($grammar_c->force_valued() < 0) {
+        Marpa::R2::uncaught_error( scalar $grammar_c->error() );
+    }
+
+    set_start_symbol($grammar);
+
+    # Catch errors in precomputation
+    my $precompute_error_code = $Marpa::R2::Error::NONE;
+    $grammar_c->throw_set(0);
+    my $precompute_result = $grammar_c->precompute();
+    $grammar_c->throw_set(1);
+
+    if ( $precompute_result < 0 ) {
+        ($precompute_error_code) = $grammar_c->error();
+        if ( not defined $precompute_error_code ) {
+            Marpa::R2::exception(
+                'libmarpa error, but no error code returned');
+        }
+
+        # If already precomputed, let higher level know
+        return $precompute_error_code
+            if $precompute_error_code == $Marpa::R2::Error::PRECOMPUTED;
+
+        # Cycles are not necessarily errors,
+        # and get special handling
+        $precompute_error_code = $Marpa::R2::Error::NONE
+            if $precompute_error_code == $Marpa::R2::Error::GRAMMAR_HAS_CYCLE;
+
+    } ## end if ( $precompute_result < 0 )
+
+    if ( $precompute_error_code != $Marpa::R2::Error::NONE ) {
+
+        # Report the errors, then return failure
+
+        if ( $precompute_error_code == $Marpa::R2::Error::NO_RULES ) {
+            Marpa::R2::exception(
+                'Attempted to precompute grammar with no rules');
+        }
+        if ( $precompute_error_code == $Marpa::R2::Error::NULLING_TERMINAL ) {
+            my @nulling_terminals = ();
+            my $event_count       = $grammar_c->event_count();
+            EVENT:
+            for ( my $event_ix = 0; $event_ix < $event_count; $event_ix++ ) {
+                my ( $event_type, $value ) = $grammar_c->event($event_ix);
+                if ( $event_type eq 'MARPA_EVENT_NULLING_TERMINAL' ) {
+                    push @nulling_terminals, $grammar->symbol_name($value);
+                }
+            } ## end EVENT: for ( my $event_ix = 0; $event_ix < $event_count; ...)
+            my @nulling_terminal_messages =
+                map {qq{Nulling symbol "$_" is also a terminal\n}}
+                @nulling_terminals;
+            Marpa::R2::exception( @nulling_terminal_messages,
+                'A terminal symbol cannot also be a nulling symbol' );
+        } ## end if ( $precompute_error_code == ...)
+        if ( $precompute_error_code == $Marpa::R2::Error::COUNTED_NULLABLE ) {
+            my @counted_nullables = ();
+            my $event_count       = $grammar_c->event_count();
+            EVENT:
+            for ( my $event_ix = 0; $event_ix < $event_count; $event_ix++ ) {
+                my ( $event_type, $value ) = $grammar_c->event($event_ix);
+                if ( $event_type eq 'MARPA_EVENT_COUNTED_NULLABLE' ) {
+                    push @counted_nullables, $grammar->symbol_name($value);
+                }
+            } ## end EVENT: for ( my $event_ix = 0; $event_ix < $event_count; ...)
+            my @counted_nullable_messages = map {
+                      q{Nullable symbol "}
+                    . $_
+                    . qq{" is on rhs of counted rule\n}
+            } @counted_nullables;
+            Marpa::R2::exception( @counted_nullable_messages,
+                'Counted nullables confuse Marpa -- please rewrite the grammar'
+            );
+        } ## end if ( $precompute_error_code == ...)
+
+        if ( $precompute_error_code == $Marpa::R2::Error::NO_START_SYMBOL ) {
+            Marpa::R2::exception('No start symbol');
+        }
+        if ( $precompute_error_code == $Marpa::R2::Error::START_NOT_LHS ) {
+            my $name = $grammar->[Marpa::R2::Internal::Grammar::START_NAME];
+            Marpa::R2::exception(
+                qq{Start symbol "$name" not on LHS of any rule});
+        }
+
+        return $precompute_error_code
+            if $precompute_error_code
+            == $Marpa::R2::Error::UNPRODUCTIVE_START;
+
+        Marpa::R2::uncaught_error( scalar $grammar_c->error() );
+
+    } ## end if ( $precompute_error_code != $Marpa::R2::Error::NONE)
+
+    # Shadow all the new rules
+    {
+        my $highest_rule_id = $grammar_c->highest_rule_id();
+        RULE:
+        for ( my $rule_id = 0; $rule_id <= $highest_rule_id; $rule_id++ ) {
+            next RULE if defined $rules->[$rule_id];
+
+            # The Marpa::R2 logic assumes no "gaps" in the rule numbering,
+            # which is currently the case for Libmarpa,
+            # but not guaranteed.
+            shadow_rule( $grammar, $rule_id );
+        } ## end RULE: for ( my $rule_id = 0; $rule_id <= $highest_rule_id; ...)
+    }
+
+    my $infinite_action =
+        $grammar->[Marpa::R2::Internal::Grammar::INFINITE_ACTION];
+
+    # Above I went through the error events
+    # Here I go through the events for situations where there was no
+    # hard error returned from libmarpa
+    my $loop_rule_count = 0;
+    {
+        my $event_count = $grammar_c->event_count();
+        EVENT:
+        for ( my $event_ix = 0; $event_ix < $event_count; $event_ix++ ) {
+            my ( $event_type, $value ) = $grammar_c->event($event_ix);
+            if ( $event_type ne 'MARPA_EVENT_LOOP_RULES' ) {
+                Marpa::R2::exception(
+                    qq{Unknown grammar precomputation event; type="$event_type"}
+                );
+            }
+            $loop_rule_count = $value;
+        } ## end EVENT: for ( my $event_ix = 0; $event_ix < $event_count; ...)
+    }
+
+    if ( $loop_rule_count and $infinite_action ne 'quiet' ) {
+        my @loop_rules =
+            grep { $grammar_c->rule_is_loop($_) } ( 0 .. $#{$rules} );
+        for my $rule_id (@loop_rules) {
+            print {$trace_fh}
+                'Cycle found involving rule: ',
+                $grammar->brief_rule($rule_id), "\n"
+                or Marpa::R2::exception("Could not print: $ERRNO");
+        } ## end for my $rule_id (@loop_rules)
+        Marpa::R2::exception('Cycles in grammar, fatal error')
+            if $infinite_action eq 'fatal';
+    } ## end if ( $loop_rule_count and $infinite_action ne 'quiet')
+
+    my $default_if_inaccessible =
+        $grammar->[Marpa::R2::Internal::Grammar::INTERNAL]->{if_inaccessible}
+        // 'warn';
+    SYMBOL:
+    for my $symbol_id ( grep { !$grammar_c->symbol_is_accessible($_) }
+        ( 0 .. $#{$symbols} ) )
+    {
+
+        my $symbol      = $symbols->[$symbol_id];
+        my $symbol_name = $grammar->symbol_name($symbol_id);
+
+        # Inaccessible internal symbols may be created
+        # from inaccessible use symbols -- ignore these.
+        # This assumes that Marpa's logic
+        # is correct and that
+        # it is not creating inaccessible symbols from
+        # accessible ones.
+        next SYMBOL if $symbol_name =~ /\]/xms;
+        my $treatment =
+            $symbol->[Marpa::R2::Internal::Symbol::IF_INACCESSIBLE] //
+            $default_if_inaccessible;
+        next SYMBOL if $treatment eq 'ok';
+        my $message = "Inaccessible symbol: $symbol_name";
+        Marpa::R2::exception($message) if $treatment eq 'fatal';
+        say {$trace_fh} $message
+            or Marpa::R2::exception("Could not print: $ERRNO");
+    } ## end for my $symbol_id ( grep { !$grammar_c->...})
+
+    # A bit hackish here: UNPRODUCTIVE_OK is not a HASH ref iff
+    # it is a Boolean TRUE indicating that all inaccessibles are OK.
+    # A Boolean FALSE will have been replaced with an empty hash.
+    if ($grammar->[Marpa::R2::Internal::Grammar::WARNINGS]
+        and ref(
+            my $ok = $grammar->[Marpa::R2::Internal::Grammar::UNPRODUCTIVE_OK]
+        ) eq 'HASH'
+        )
+    {
+        SYMBOL:
+        for my $symbol (
+            @{ Marpa::R2::Grammar::unproductive_symbols($grammar) } )
+        {
+
+            # Unproductive internal symbols may be created
+            # from unproductive use symbols -- ignore these.
+            # This assumes that Marpa's logic
+            # is correct and that
+            # it is not creating unproductive symbols from
+            # productive ones.
+            next SYMBOL if $symbol =~ /\]/xms;
+            next SYMBOL if $ok->{$symbol};
+            say {$trace_fh} "Unproductive symbol: $symbol"
+                or Marpa::R2::exception("Could not print: $ERRNO");
+        } ## end SYMBOL: for my $symbol ( @{ ...})
+    } ## end if ( $grammar->[Marpa::R2::Internal::Grammar::WARNINGS...])
+
+    my $cc_hash = $grammar->[Marpa::R2::Internal::Grammar::CHARACTER_CLASSES];
+    if ( defined $cc_hash ) {
+        my $class_table =
+            $grammar->[Marpa::R2::Internal::Grammar::CHARACTER_CLASS_TABLE] =
+            [];
+        for my $cc_symbol ( sort keys %{$cc_hash} ) {
+            my $cc_components = $cc_hash->{$cc_symbol};
+            push @{$class_table},
+                [ $grammar->thin_symbol($cc_symbol), $cc_components ];
+        }
+    } ## end if ( defined $cc_hash )
+
+    # Save some memory
+    $grammar->[Marpa::R2::Internal::Grammar::CHARACTER_CLASSES] = undef;
+
+    return ;
+
+} ## end sub Marpa::R2::Grammar::slif_precompute
 
 sub Marpa::R2::Grammar::show_problems {
     my ($grammar) = @_;
@@ -675,10 +886,9 @@ sub Marpa::R2::Grammar::symbol_in_display_form {
     return "<!No symbol with ID $symbol_id!>" if not defined $symbol;
     my $text = $symbol->[Marpa::R2::Internal::Symbol::DISPLAY_FORM];
     return $text if defined $text;
-    $text = $symbol->[Marpa::R2::Internal::Symbol::DSL_FORM];
-    return "<$text>" if defined $text;
-    $text = $grammar->symbol_name($symbol_id);
-    return "<$text>";
+    $text = $symbol->[Marpa::R2::Internal::Symbol::DSL_FORM] //
+     $grammar->symbol_name($symbol_id);
+    return ($text =~ m/\s/xms) ? "<$text>" : $text;
 }
 
 sub Marpa::R2::Grammar::show_symbol {
@@ -760,6 +970,33 @@ sub Marpa::R2::Grammar::unproductive_symbols {
     ];
 } ## end sub Marpa::R2::Grammar::unproductive_symbols
 
+sub Marpa::R2::Grammar::start_symbol {
+    my ( $grammar ) = @_;
+    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
+    return $grammar_c->start_symbol();
+}
+
+sub Marpa::R2::Grammar::rule_name {
+    my ( $grammar, $rule_id ) = @_;
+    my $rules = $grammar->[Marpa::R2::Internal::Grammar::RULES];
+    my $rule  = $rules->[$rule_id];
+    return "Non-existent rule $rule_id" if not defined $rule;
+    my $name = $rule->[Marpa::R2::Internal::Rule::NAME];
+    return $name if defined $name;
+    my $tracer    = $grammar->[Marpa::R2::Internal::Grammar::TRACER];
+    my ( $lhs_id ) = $tracer->rule_expand($rule_id);
+    return $grammar->symbol_name($lhs_id);
+} ## end sub Marpa::R2::Grammar::rule_name
+
+# Undocumented -- assumes it is called internally,
+# by the SLIF
+sub Marpa::R2::Grammar::tag {
+    my ( $grammar, $rule_id ) = @_;
+    my $rules = $grammar->[Marpa::R2::Internal::Grammar::RULES];
+    my $rule  = $rules->[$rule_id];
+    return $rule->[Marpa::R2::Internal::Rule::SLIF_TAG];
+} ## end sub Marpa::R2::Grammar::rule_name
+
 sub Marpa::R2::Grammar::brief_rule {
     my ( $grammar, $rule_id ) = @_;
     my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
@@ -822,13 +1059,15 @@ sub Marpa::R2::Grammar::symbol_ids {
     return 0 .. $grammar_c->highest_symbol_id();
 } ## end sub Marpa::R2::Grammar::rule_ids
 
+# Returns empty array if not such rule
 sub Marpa::R2::Grammar::rule {
     my ( $grammar, $rule_id ) = @_;
     my $symbols     = $grammar->[Marpa::R2::Internal::Grammar::SYMBOLS];
     my $tracer    = $grammar->[Marpa::R2::Internal::Grammar::TRACER];
     my @symbol_names = ();
 
-    SYMBOL_ID: for my $symbol_id ($tracer->rule_expand($rule_id)) {
+    my @symbols = $tracer->rule_expand($rule_id);
+    SYMBOL_ID: for my $symbol_id (@symbols) {
         ## The name of the symbols, before the BNF rewrites
         my $name =
             $symbols->[$symbol_id]->[Marpa::R2::Internal::Symbol::LEGACY_NAME]
@@ -965,6 +1204,11 @@ sub assign_symbol {
             $symbol->[Marpa::R2::Internal::Symbol::DISPLAY_FORM] = $value;
             next PROPERTY;
         }
+        if ( $property eq 'if_inaccessible' ) {
+            my $value = $options->{$property};
+            $symbol->[Marpa::R2::Internal::Symbol::IF_INACCESSIBLE] = $value;
+            next PROPERTY;
+        }
         Marpa::R2::exception(qq{Unknown symbol property "$property"});
     } ## end PROPERTY: for my $property ( keys %{$options} )
 
@@ -1052,10 +1296,10 @@ sub add_user_rule {
 
     Marpa::R2::exception('Missing argument to add_user_rule')
         if not defined $grammar
-            or not defined $options;
+        or not defined $options;
 
     my $grammar_c    = $grammar->[Marpa::R2::Internal::Grammar::C];
-    my $tracer    = $grammar->[Marpa::R2::Internal::Grammar::TRACER];
+    my $tracer       = $grammar->[Marpa::R2::Internal::Grammar::TRACER];
     my $rules        = $grammar->[Marpa::R2::Internal::Grammar::RULES];
     my $default_rank = $grammar_c->default_rank();
 
@@ -1064,17 +1308,20 @@ sub add_user_rule {
     my $rank;
     my $null_ranking;
     my $rule_name;
+    my $slif_tag;
     my $mask;
     my $proper_separation = 0;
     my $keep_separation   = 0;
     my $description;
 
-    OPTION: while ( my ( $option, $value ) = each %{$options} ) {
+    OPTION: for my $option ( keys %{$options} ) {
+        my $value = $options->{$option};
         if ( $option eq 'name' )   { $rule_name = $value; next OPTION; }
+        if ( $option eq 'tag' )    { $slif_tag  = $value; next OPTION; }
         if ( $option eq 'rhs' )    { $rhs_names = $value; next OPTION }
         if ( $option eq 'lhs' )    { $lhs_name  = $value; next OPTION }
         if ( $option eq 'action' ) { $action    = $value; next OPTION }
-        if ( $option eq 'bless' ) { $blessing    = $value; next OPTION }
+        if ( $option eq 'bless' )  { $blessing  = $value; next OPTION }
         if ( $option eq 'rank' )   { $rank      = $value; next OPTION }
         if ( $option eq 'null_ranking' ) {
             $null_ranking = $value;
@@ -1090,10 +1337,10 @@ sub add_user_rule {
             next OPTION;
         }
         if ( $option eq 'keep' ) { $keep_separation = $value; next OPTION }
-        if ( $option eq 'mask' ) { $mask = $value; next OPTION }
+        if ( $option eq 'mask' ) { $mask            = $value; next OPTION }
         if ( $option eq 'description' ) { $description = $value; next OPTION }
         Marpa::R2::exception("Unknown user rule option: $option");
-    } ## end OPTION: while ( my ( $option, $value ) = each %{$options} )
+    } ## end OPTION: for my $option ( keys %{$options} )
 
     if ( defined $min and not Scalar::Util::looks_like_number($min) ) {
         Marpa::R2::exception(
@@ -1102,8 +1349,8 @@ sub add_user_rule {
     my $stuifzand_interface =
         $grammar->[Marpa::R2::Internal::Grammar::INTERFACE] eq 'stuifzand';
 
-    my $grammar_is_internal = $stuifzand_interface ||
-        $grammar->[Marpa::R2::Internal::Grammar::INTERNAL];
+    my $grammar_is_internal = $stuifzand_interface
+        || $grammar->[Marpa::R2::Internal::Grammar::INTERNAL];
 
     my $lhs =
         $grammar_is_internal
@@ -1142,23 +1389,6 @@ sub add_user_rule {
             "Null Ranking must be undefined, 'high' or 'low'\n";
     }
 
-    # Determine the rule's name
-    my $rules_by_name =
-        $grammar->[Marpa::R2::Internal::Grammar::RULE_BY_NAME];
-    if ( defined $rule_name and defined $rules_by_name->{$rule_name} ) {
-        push @rule_problems, qq{rule named "$rule_name" already exists};
-    }
-    if ( !defined $rule_name
-        and $grammar->[Marpa::R2::Internal::Grammar::RULE_NAME_REQUIRED] )
-    {
-        $rule_name =
-            $grammar->symbol_name( $lhs->[Marpa::R2::Internal::Symbol::ID] );
-        if ( defined $rules_by_name->{$rule_name} ) {
-            push @rule_problems,
-                qq{Cannot name rule from LHS; rule named "$rule_name" already exists};
-        }
-    } ## end if ( !defined $rule_name and $grammar->[...])
-
     if ( scalar @rule_problems ) {
         my %dump_options = %{$options};
         delete $dump_options{grammar};
@@ -1195,81 +1425,50 @@ sub add_user_rule {
     my @rhs_ids = map { $_->[Marpa::R2::Internal::Symbol::ID] } @{$rhs};
     my $lhs_id = $lhs->[Marpa::R2::Internal::Symbol::ID];
 
+    my $base_rule_id;
+    my $separator_id = -1;
+
     if ($is_ordinary_rule) {
 
         # Capture errors
         $grammar_c->throw_set(0);
-        my $ordinary_rule_id = $grammar_c->rule_new( $lhs_id, \@rhs_ids );
+        $base_rule_id = $grammar_c->rule_new( $lhs_id, \@rhs_ids );
         $grammar_c->throw_set(1);
 
-        if ( $ordinary_rule_id < 0 ) {
-            my $rule_description =
-                "$lhs_name -> " . ( join q{ }, @{$rhs_names} );
-            my ( $error_code, $error_string ) = $grammar_c->error();
-            $error_code //= -1;
-            my $problem_description =
-                $error_code == $Marpa::R2::Error::DUPLICATE_RULE
-                ? 'Duplicate rule'
-                : $error_string;
-            Marpa::R2::exception("$problem_description: $rule_description");
-        } ## end if ( $ordinary_rule_id < 0 )
-        shadow_rule( $grammar, $ordinary_rule_id );
-        my $ordinary_rule = $rules->[$ordinary_rule_id];
+    } ## end if ($is_ordinary_rule)
+    else {
+        Marpa::R2::exception('Only one rhs symbol allowed for counted rule')
+            if scalar @{$rhs_names} != 1;
 
-        # Only internal grammars can set a custom mask
-        if ( not defined $mask or not $grammar_is_internal ) {
-            $mask = [ (1) x scalar @rhs_ids ];
-        }
-        $ordinary_rule->[Marpa::R2::Internal::Rule::MASK] = $mask;
-        $ordinary_rule->[Marpa::R2::Internal::Rule::ACTION_NAME] = $action;
+        # create the separator symbol, if we're using one
+        if ( defined $separator_name ) {
+            my $separator =
+                $grammar_is_internal
+                ? assign_symbol( $grammar, $separator_name )
+                : assign_user_symbol( $grammar, $separator_name );
+            $separator_id = $separator->[Marpa::R2::Internal::Symbol::ID];
+        } ## end if ( defined $separator_name )
 
-        if ( defined $rank ) {
-            $grammar_c->rule_rank_set( $ordinary_rule_id, $rank );
-        }
-        $grammar_c->rule_null_high_set( $ordinary_rule_id,
-            ( $null_ranking eq 'high' ? 1 : 0 ) );
-        if ( defined $rule_name ) {
-            $ordinary_rule->[Marpa::R2::Internal::Rule::NAME] = $rule_name;
-            $rules_by_name->{$rule_name} = $ordinary_rule;
-        }
+        $grammar_c->throw_set(0);
 
-        if ( defined $blessing ) {
-            $ordinary_rule->[Marpa::R2::Internal::Rule::BLESSING] = $blessing;
-        }
+        # The original rule for a sequence rule is
+        # not actually used in parsing,
+        # but some of the rewritten sequence rules are its
+        # semantic equivalents.
 
-        if ( defined $description ) {
-            $ordinary_rule->[Marpa::R2::Internal::Rule::DESCRIPTION] = $description;
-        }
+        $base_rule_id = $grammar_c->sequence_new(
+            $lhs_id,
+            $rhs_ids[0],
+            {   separator => $separator_id,
+                proper    => $proper_separation,
+                min       => $min,
+            }
+        );
+        $grammar_c->throw_set(1);
+    } ## end else [ if ($is_ordinary_rule) ]
 
-        return;
-    }    # not defined $min
-
-    Marpa::R2::exception('Only one rhs symbol allowed for counted rule')
-        if scalar @{$rhs_names} != 1;
-
-    # create the separator symbol, if we're using one
-    my $separator;
-    my $separator_id = -1;
-    if ( defined $separator_name ) {
-        $separator =
-            $grammar_is_internal
-            ? assign_symbol( $grammar, $separator_name )
-            : assign_user_symbol( $grammar, $separator_name );
-        $separator_id = $separator->[Marpa::R2::Internal::Symbol::ID];
-    } ## end if ( defined $separator_name )
-
-    $grammar_c->throw_set(0);
-    my $original_rule_id = $grammar_c->sequence_new(
-        $lhs_id,
-        $rhs_ids[0],
-        {   separator => $separator_id,
-            proper    => $proper_separation,
-            min       => $min,
-        }
-    );
-    $grammar_c->throw_set(1);
-    if ( not defined $original_rule_id or $original_rule_id < 0) {
-        my $rule_description = "$lhs_name -> " . ( join q{ }, @{$rhs_names} );
+    if ( not defined $base_rule_id or $base_rule_id < 0 ) {
+        my $rule_description = rule_describe( $lhs_name, $rhs_names );
         my ( $error_code, $error_string ) = $grammar_c->error();
         $error_code //= -1;
         my $problem_description =
@@ -1277,37 +1476,58 @@ sub add_user_rule {
             ? 'Duplicate rule'
             : $error_string;
         Marpa::R2::exception("$problem_description: $rule_description");
-    } ## end if ( not defined $original_rule_id )
+    } ## end if ( not defined $base_rule_id or $base_rule_id < 0 )
 
-    shadow_rule( $grammar, $original_rule_id );
+    my $base_rule = shadow_rule( $grammar, $base_rule_id );
 
-    # The original rule for a sequence rule is
-    # not actually used in parsing,
-    # but some of the rewritten sequence rules are its
-    # semantic equivalents.
-    my $original_rule = $rules->[$original_rule_id];
-    $original_rule->[Marpa::R2::Internal::Rule::ACTION_NAME] = $action;
-    $original_rule->[Marpa::R2::Internal::Rule::DISCARD_SEPARATION] =
+    if ($is_ordinary_rule) {
+
+        # Only internal grammars can set a custom mask
+        if ( not defined $mask or not $grammar_is_internal ) {
+            $mask = [ (1) x scalar @rhs_ids ];
+        }
+        $base_rule->[Marpa::R2::Internal::Rule::MASK] = $mask;
+    } ## end if ($is_ordinary_rule)
+
+    $base_rule->[Marpa::R2::Internal::Rule::DISCARD_SEPARATION] =
         $separator_id >= 0 && !$keep_separation;
-    $grammar_c->rule_null_high_set( $original_rule_id,
+
+    $base_rule->[Marpa::R2::Internal::Rule::ACTION_NAME] = $action;
+    $grammar_c->rule_null_high_set( $base_rule_id,
         ( $null_ranking eq 'high' ? 1 : 0 ) );
-    $grammar_c->rule_rank_set( $original_rule_id, $rank );
+    $grammar_c->rule_rank_set( $base_rule_id, $rank );
 
     if ( defined $rule_name ) {
-        $original_rule->[Marpa::R2::Internal::Rule::NAME] = $rule_name;
-        $rules_by_name->{$rule_name} = $original_rule;
+        $base_rule->[Marpa::R2::Internal::Rule::NAME] = $rule_name;
     }
+    if ( defined $slif_tag ) {
+        $base_rule->[Marpa::R2::Internal::Rule::SLIF_TAG] = $slif_tag;
+        my $rule_id_by_tag =
+            $grammar->[Marpa::R2::Internal::Grammar::RULE_ID_BY_TAG];
+        if ( $rule_id_by_tag->{$slif_tag} ) {
+            Marpa::R2::exception(
+                qq{Duplicate tag in SLIF rule, tag was "$slif_tag"});
+        }
+        $rule_id_by_tag->{$slif_tag} = $base_rule_id;
+    } ## end if ( defined $slif_tag )
     if ( defined $blessing ) {
-        $original_rule->[Marpa::R2::Internal::Rule::BLESSING] = $blessing;
+        $base_rule->[Marpa::R2::Internal::Rule::BLESSING] = $blessing;
     }
     if ( defined $description ) {
-        $original_rule->[Marpa::R2::Internal::Rule::DESCRIPTION] = $description;
+        $base_rule->[Marpa::R2::Internal::Rule::DESCRIPTION] = $description;
     }
 
     return;
 
 } ## end sub add_user_rule
 
+sub rule_describe {
+    my ( $lhs_name, $rhs_names ) = @_;
+    # wrap symbol names with whitespaces allowed by SLIF
+    $lhs_name = "<$lhs_name>" if $lhs_name =~ / /;
+    return "$lhs_name -> " . ( join q{ }, map { / / ? "<$_>" : $_ } @{$rhs_names} );
+} ## end sub rule_describe
+
 sub set_start_symbol {
     my $grammar = shift;
 
@@ -1365,7 +1585,7 @@ sub Marpa::R2::Grammar::show_isy {
     $text .= "$isy_id: $name";
 
     my @tag_list = ();
-    $grammar_c->_marpa_g_isy_is_nulling($isy_id)
+    $grammar_c->_marpa_g_nsy_is_nulling($isy_id)
         and push @tag_list, 'nulling';
 
     $text .= join q{ }, q{,}, @tag_list if scalar @tag_list;
@@ -1379,7 +1599,7 @@ sub Marpa::R2::Grammar::show_isys {
     my ($grammar) = @_;
     my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
     my $text      = q{};
-    for my $isy_id ( 0 .. $grammar_c->_marpa_g_isy_count() - 1 ) {
+    for my $isy_id ( 0 .. $grammar_c->_marpa_g_nsy_count() - 1 ) {
         $text .= $grammar->show_isy($isy_id);
     }
     return $text;
@@ -1417,16 +1637,10 @@ sub Marpa::R2::Grammar::rule_is_used {
     return $grammar_c->_marpa_g_rule_is_used($rule_id);
 }
 
-sub Marpa::R2::Grammar::show_AHFA {
+sub Marpa::R2::Grammar::show_ahms {
     my ( $grammar, $verbose ) = @_;
     return $grammar->[Marpa::R2::Internal::Grammar::TRACER]
-        ->show_AHFA($verbose);
-}
-
-sub Marpa::R2::Grammar::show_AHFA_items {
-    my ( $grammar, $verbose ) = @_;
-    return $grammar->[Marpa::R2::Internal::Grammar::TRACER]
-        ->show_AHFA_items($verbose);
+        ->show_ahms($verbose);
 }
 
 1;
