@@ -1,4 +1,4 @@
-# Copyright 2013 Jeffrey Kegler
+# Copyright 2015 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -21,7 +21,7 @@ use strict;
 use English qw( -no_match_vars );
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '2.073_001';
+$VERSION        = '2.103_008';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -39,14 +39,26 @@ sub Marpa::R2::Recognizer::new {
     my $recce = bless [], $class;
 
     my $grammar;
-    ARG_HASH: for my $arg_hash (@arg_hashes) {
-        if ( defined( $grammar = $arg_hash->{grammar} ) ) {
-            delete $arg_hash->{grammar};
-            last ARG_HASH;
+    my $trace_file_handle;
+    for my $arg_hash (@arg_hashes) {
+
+        # Need to capture the trace file handle early
+        my $value;
+        if ( defined( $value = $arg_hash->{trace_file_handle} ) ) {
+            delete $arg_hash->{trace_file_handle};
+            $trace_file_handle = $value;
         }
-    } ## end ARG_HASH: for my $arg_hash (@arg_hashes)
+        if ( defined( $value = $arg_hash->{grammar} ) ) {
+            delete $arg_hash->{grammar};
+            $grammar = $value;
+        }
+    } ## end for my $arg_hash (@arg_hashes)
     Marpa::R2::exception('No grammar specified') if not defined $grammar;
 
+    $trace_file_handle //= $grammar->[Marpa::R2::Internal::Grammar::TRACE_FILE_HANDLE] ;
+    local $Marpa::R2::Internal::TRACE_FH =
+        $recce->[Marpa::R2::Internal::Recognizer::TRACE_FILE_HANDLE] = $trace_file_handle;
+
     $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR] = $grammar;
 
     my $grammar_class = ref $grammar;
@@ -66,11 +78,6 @@ sub Marpa::R2::Recognizer::new {
         );
     } ## end if ($problems)
 
-    # set the defaults
-    local $Marpa::R2::Internal::TRACE_FH = my $trace_fh =
-        $recce->[Marpa::R2::Internal::Recognizer::TRACE_FILE_HANDLE] =
-        $grammar->[Marpa::R2::Internal::Grammar::TRACE_FILE_HANDLE];
-
     my $recce_c = $recce->[Marpa::R2::Internal::Recognizer::C] =
         Marpa::R2::Thin::R->new($grammar_c);
     if ( not defined $recce_c ) {
@@ -119,6 +126,7 @@ sub Marpa::R2::Recognizer::new {
     $recce->[Marpa::R2::Internal::Recognizer::WARNINGS]       = 1;
     $recce->[Marpa::R2::Internal::Recognizer::RANKING_METHOD] = 'none';
     $recce->[Marpa::R2::Internal::Recognizer::MAX_PARSES]     = 0;
+    $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS]     = 0;
 
     # Position 0 is not used because 0 indicates an unvalued token.
     # Position 1 is reserved for undef.
@@ -135,19 +143,14 @@ sub Marpa::R2::Recognizer::new {
 
     $recce->set(@arg_hashes);
 
-    my $trace_terminals =
-        $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS] // 0;
-    my $trace_tasks = $recce->[Marpa::R2::Internal::Recognizer::TRACE_TASKS]
-        // 0;
-
-    if ( $trace_terminals > 1 ) {
+    if ( $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS] > 1 ) {
         my @terminals_expected = @{ $recce->terminals_expected() };
         for my $terminal ( sort @terminals_expected ) {
             say {$Marpa::R2::Internal::TRACE_FH}
                 qq{Expecting "$terminal" at earleme 0}
                 or Marpa::R2::exception("Cannot print: $ERRNO");
         }
-    } ## end if ( $trace_terminals > 1 )
+    } ## end if ( $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS...])
 
     return $recce;
 } ## end sub Marpa::R2::Recognizer::new
@@ -161,6 +164,16 @@ sub Marpa::R2::Recognizer::thin {
     $_[0]->[Marpa::R2::Internal::Recognizer::C];
 }
 
+# For the non-legacy case,
+# I reset the ordering, forcing it to be recalculated
+# for each parse series.
+# But I do not actually allow the ranking method to
+# be changed once a parse is created.
+# Since I am stabilizing Marpa::R2, the "fix" should
+# probably be to save the overhead, rather than
+# to allow 'ranking_method' to be changed.
+# But for now I will do nothing.
+# JK -- Mon Nov 24 17:35:24 PST 2014
 sub Marpa::R2::Recognizer::reset_evaluation {
     my ($recce) = @_;
     my $grammar = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
@@ -211,39 +224,38 @@ sub Marpa::R2::Recognizer::set {
             );
         } ## end if ( not $ref_type or $ref_type ne 'HASH' )
 
-	state $recognizer_options = {
-	    map { ( $_, 1 ) }
-		qw(
-		closures
-		end
-		event_if_expected
-		leo
-		max_parses
-		semantics_package
-		ranking_method
-		too_many_earley_items
-		trace_actions
-		trace_and_nodes
-		trace_bocage
-		trace_earley_sets
-		trace_fh
-		trace_file_handle
-		trace_or_nodes
-		trace_tasks
-		trace_terminals
-		trace_values
-		warnings
-		)
-	};
+        state $recognizer_options = {
+            map { ( $_, 1 ) }
+                qw(
+                closures
+                end
+                event_if_expected
+                leo
+                max_parses
+                semantics_package
+                ranking_method
+                too_many_earley_items
+                trace_actions
+                trace_and_nodes
+                trace_bocage
+                trace_earley_sets
+                trace_fh
+                trace_file_handle
+                trace_or_nodes
+                trace_terminals
+                trace_values
+                warnings
+                )
+        };
 
-	if (my @bad_options =
-	    grep { not exists $recognizer_options->{$_} }
-	    keys %{$args}
-	    )
-	{
-	    Carp::croak( 'Unknown option(s) for Marpa::R2 Recognizer: ',
-		join q{ }, @bad_options );
-	} ## end if ( my @bad_options = grep { not exists $recognizer_options...})
+        if (my @bad_options =
+            grep { not exists $recognizer_options->{$_} }
+            keys %{$args}
+            )
+        {
+            Carp::croak( 'Unknown option(s) for Marpa::R2 Recognizer: ',
+                join q{ }, @bad_options );
+        } ## end if ( my @bad_options = grep { not exists $recognizer_options...})
 
         if ( defined( my $value = $args->{'event_if_expected'} ) ) {
             ## It could be allowed, but it is not needed and this is simpler
@@ -356,20 +368,9 @@ sub Marpa::R2::Recognizer::set {
             }
         } ## end if ( defined( my $value = $args->{'trace_or_nodes'} ...))
 
-        if ( defined( my $value = $args->{'trace_tasks'} ) ) {
-            Marpa::R2::exception('trace_tasks must be set to a number >= 0')
-                if $value !~ /\A\d+\z/xms;
-            $recce->[Marpa::R2::Internal::Recognizer::TRACE_TASKS] =
-                $value + 0;
-            if ($value) {
-                say {$trace_fh} "Setting trace_tasks option to $value"
-                    or Marpa::R2::exception("Cannot print: $ERRNO");
-            }
-        } ## end if ( defined( my $value = $args->{'trace_tasks'} ) )
-
         if ( defined( my $value = $args->{'trace_terminals'} ) ) {
             $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS] =
-                $value;
+                Scalar::Util::looks_like_number($value) ? $value : 0;
             if ($value) {
                 say {$trace_fh} 'Setting trace_terminals option'
                     or Marpa::R2::exception("Cannot print: $ERRNO");
@@ -412,7 +413,8 @@ sub Marpa::R2::Recognizer::set {
             }
             my $closures =
                 $recce->[Marpa::R2::Internal::Recognizer::CLOSURES] = $value;
-            while ( my ( $action, $closure ) = each %{$closures} ) {
+            for my $action ( keys %{$closures} ) {
+                my $closure = $closures->{$action};
                 Marpa::R2::exception(qq{Bad closure for action "$action"})
                     if ref $closure ne 'CODE';
             }
@@ -662,9 +664,9 @@ sub Marpa::R2::Recognizer::alternative {
         {
             Marpa::R2::exception('alternative(): value must be undef or ref');
         } ## end if ( $ref_type ne 'SCALAR' and $ref_type ne 'REF' and...)
+        my $value = ${$value_ref};
+        last SET_VALUE_IX if not defined $value;
         $value_ix = scalar @{$token_values};
-	my $value = ${$value_ref};
-	last SET_VALUE_IX if not defined $value;
         push @{$token_values}, $value;
     } ## end SET_VALUE_IX:
     $length //= 1;
@@ -772,7 +774,7 @@ sub Marpa::R2::Recognizer::earleme_complete {
     } ## end if ( $recce->[Marpa::R2::Internal::Recognizer::TRACE_EARLEY_SETS...])
 
     my $trace_terminals =
-        $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS] // 0;
+        $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS];
     if ( $trace_terminals > 1 ) {
         my $current_earleme    = $recce_c->current_earleme();
         my $terminals_expected = $recce->terminals_expected();
@@ -843,6 +845,17 @@ sub Marpa::R2::Recognizer::earley_set_size {
     return $recce_c->_marpa_r_earley_set_size($set_id);
 }
 
+sub ahm_describe {
+    my ($recce, $ahm_id)        = @_;
+    my $recce_c        = $recce->[Marpa::R2::Internal::Recognizer::C];
+    my $grammar        = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
+    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
+    my $irl_id = $grammar_c->_marpa_g_ahm_irl($ahm_id);
+    my $dot_position = $grammar_c->_marpa_g_ahm_position($ahm_id);
+    if ($dot_position < 0) { return 'R' . $irl_id . q{$} }
+    return 'R' . $irl_id . q{:} . $dot_position;
+}
+
 sub Marpa::R2::show_leo_item {
     my ($recce)        = @_;
     my $recce_c        = $recce->[Marpa::R2::Internal::Recognizer::C];
@@ -876,24 +889,26 @@ sub Marpa::R2::show_token_link_choice {
     my ( $recce, $current_earleme ) = @_;
     my $recce_c = $recce->[Marpa::R2::Internal::Recognizer::C];
     my $grammar = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
+    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
     my $tracer  = $grammar->[Marpa::R2::Internal::Grammar::TRACER];
     my $text    = q{};
     my @pieces  = ();
     my ( $token_id, $value_ix ) = $recce_c->_marpa_r_source_token();
-    my $predecessor_state = $recce_c->_marpa_r_source_predecessor_state();
+    my $predecessor_ahm = $recce_c->_marpa_r_source_predecessor_state();
     my $origin_set_id     = $recce_c->_marpa_r_earley_item_origin();
     my $origin_earleme    = $recce_c->earleme($origin_set_id);
     my $middle_earleme    = $origin_earleme;
 
-    if ( defined $predecessor_state ) {
+    if ( defined $predecessor_ahm ) {
         my $middle_set_id = $recce_c->_marpa_r_source_middle();
         $middle_earleme = $recce_c->earleme($middle_set_id);
         push @pieces,
-              'p=S'
-            . $predecessor_state . q{@}
+              'c='
+            . ahm_describe($recce, $predecessor_ahm)
+            . q{@}
             . $origin_earleme . q{-}
             . $middle_earleme;
-    } ## end if ( defined $predecessor_state )
+    } ## end if ( defined $predecessor_ahm )
     my $symbol_name = $tracer->isy_name($token_id);
     push @pieces, 's=' . $symbol_name;
     my $token_length = $current_earleme - $middle_earleme;
@@ -907,9 +922,10 @@ sub Marpa::R2::show_token_link_choice {
 
 # Assumes trace completion source link set by caller
 sub Marpa::R2::show_completion_link_choice {
-    my ( $recce, $AHFA_state_id, $current_earleme ) = @_;
+    my ( $recce, $link_ahm_id, $current_earleme ) = @_;
     my $recce_c = $recce->[Marpa::R2::Internal::Recognizer::C];
     my $grammar = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
+    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
     my $symbols = $grammar->[Marpa::R2::Internal::Grammar::SYMBOLS];
     my $text    = q{};
     my @pieces  = ();
@@ -921,14 +937,13 @@ sub Marpa::R2::show_completion_link_choice {
 
     if ( defined $predecessor_state ) {
         push @pieces,
-              'p=S'
-            . $predecessor_state . q{@}
+              'p='
+            . ahm_describe($recce, $predecessor_state) . q{@}
             . $origin_earleme . q{-}
             . $middle_earleme;
     } ## end if ( defined $predecessor_state )
     push @pieces,
-          'c=S'
-        . $AHFA_state_id . q{@}
+          'c=' . ahm_describe($recce, $link_ahm_id) . q{@}
         . $middle_earleme . q{-}
         . $current_earleme;
     return '[' . ( join '; ', @pieces ) . ']';
@@ -936,9 +951,10 @@ sub Marpa::R2::show_completion_link_choice {
 
 # Assumes trace completion source link set by caller
 sub Marpa::R2::show_leo_link_choice {
-    my ( $recce, $AHFA_state_id, $current_earleme ) = @_;
+    my ( $recce, $link_ahm_id, $current_earleme ) = @_;
     my $recce_c        = $recce->[Marpa::R2::Internal::Recognizer::C];
     my $grammar        = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
+    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
     my $symbols        = $grammar->[Marpa::R2::Internal::Grammar::SYMBOLS];
     my $text           = q{};
     my @pieces         = ();
@@ -948,8 +964,8 @@ sub Marpa::R2::show_leo_link_choice {
         $recce_c->_marpa_r_source_leo_transition_symbol();
     push @pieces, 'l=L' . $leo_transition_symbol . q{@} . $middle_earleme;
     push @pieces,
-          'c=S'
-        . $AHFA_state_id . q{@}
+          'c=' . ahm_describe($recce, $link_ahm_id)
+        . q{@}
         . $middle_earleme . q{-}
         . $current_earleme;
     return '[' . ( join '; ', @pieces ) . ']';
@@ -957,14 +973,29 @@ sub Marpa::R2::show_leo_link_choice {
 
 # Assumes trace earley item was set by caller
 sub Marpa::R2::show_earley_item {
-    my ( $recce, $current_es, $state_id ) = @_;
+    my ( $recce, $current_es, $item_id ) = @_;
     my $recce_c        = $recce->[Marpa::R2::Internal::Recognizer::C];
+    my $grammar   = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
+    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
+    my $tracer    = $grammar->[Marpa::R2::Internal::Grammar::TRACER];
+
+    my $ahm_id_of_yim = $recce_c->_marpa_r_earley_item_trace($item_id);
+    return if not defined $ahm_id_of_yim;
+
     my $text           = q{};
     my $origin_set_id  = $recce_c->_marpa_r_earley_item_origin();
     my $earleme        = $recce_c->earleme($current_es);
     my $origin_earleme = $recce_c->earleme($origin_set_id);
-    $text .= sprintf "S%d@%d-%d", $state_id, $origin_earleme, $earleme;
-    my @pieces    = $text;
+    $text .= sprintf "ahm%d: %s@%d-%d", $ahm_id_of_yim,
+        ahm_describe($recce, $ahm_id_of_yim),
+        $origin_earleme, $earleme;
+    my @lines    = $text;
+    my $irl_id = $grammar_c->_marpa_g_ahm_irl($ahm_id_of_yim);
+    my $dot_position = $grammar_c->_marpa_g_ahm_position($ahm_id_of_yim);
+    push @lines, qq{  }
+        . ahm_describe($recce, $ahm_id_of_yim)
+        . q{: }
+        . $tracer->show_dotted_irl($irl_id, $dot_position);
     my @sort_data = ();
 
     for (
@@ -981,7 +1012,7 @@ sub Marpa::R2::show_earley_item {
             Marpa::R2::show_token_link_choice( $recce, $earleme )
             ];
     } ## end for ( my $symbol_id = $recce_c->_marpa_r_first_token_link_trace...)
-    push @pieces, map { $_->[-1] } sort {
+    my @pieces = map { $_->[-1] } sort {
                $a->[0] <=> $b->[0]
             || $a->[1] <=> $b->[1]
             || $a->[2] <=> $b->[2]
@@ -1010,50 +1041,45 @@ sub Marpa::R2::show_earley_item {
     } @sort_data;
     @sort_data = ();
     for (
-        my $AHFA_state_id = $recce_c->_marpa_r_first_leo_link_trace();
-        defined $AHFA_state_id;
-        $AHFA_state_id = $recce_c->_marpa_r_next_leo_link_trace()
+        my $link_ahm_id = $recce_c->_marpa_r_first_leo_link_trace();
+        defined $link_ahm_id;
+        $link_ahm_id = $recce_c->_marpa_r_next_leo_link_trace()
         )
     {
         push @sort_data,
             [
             $recce_c->_marpa_r_source_middle(),
-            $AHFA_state_id,
+            $link_ahm_id,
             $recce_c->_marpa_r_source_leo_transition_symbol(),
             Marpa::R2::show_leo_link_choice(
-                $recce, $AHFA_state_id, $earleme
+                $recce, $link_ahm_id, $earleme
             )
             ];
-    } ## end for ( my $AHFA_state_id = $recce_c...)
+    } ## end for ( my $link_ahm_id = $recce_c...)
     push @pieces, map { $_->[-1] } sort {
                $a->[0] <=> $b->[0]
             || $a->[1] <=> $b->[1]
             || $a->[2] <=> $b->[2]
     } @sort_data;
-    return join q{ }, @pieces;
+    push @lines, q{  } . join q{ }, @pieces if @pieces;
+    return join "\n", @lines, q{};
 } ## end sub Marpa::R2::show_earley_item
 
 sub Marpa::R2::show_earley_set {
     my ( $recce, $traced_set_id ) = @_;
     my $recce_c   = $recce->[Marpa::R2::Internal::Recognizer::C];
     my $text      = q{};
-    my @sort_data = ();
+    my @sorted_data = ();
     if ( not defined $recce_c->_marpa_r_earley_set_trace($traced_set_id) ) {
         return $text;
     }
     EARLEY_ITEM: for ( my $item_id = 0;; $item_id++ ) {
-        my $state_id = $recce_c->_marpa_r_earley_item_trace($item_id);
-        last EARLEY_ITEM if not defined $state_id;
-        push @sort_data,
-            [
-            $recce_c->_marpa_r_earley_item_origin(), $state_id,
-            Marpa::R2::show_earley_item( $recce, $traced_set_id, $state_id )
-            ];
+        my $item_desc = Marpa::R2::show_earley_item( $recce, $traced_set_id, $item_id );
+        last EARLEY_ITEM if not defined $item_desc;
+        # We do not sort these any more
+        push @sorted_data, $item_desc;
     } ## end EARLEY_ITEM: for ( my $item_id = 0;; $item_id++ )
-    my @sorted_data =
-        map { $_->[-1] . "\n" }
-        sort { $a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] } @sort_data;
-    @sort_data = ();
+    my @sort_data = ();
     POSTDOT_ITEM:
     for (
         my $postdot_symbol_id = $recce_c->_marpa_r_first_postdot_item_trace();
@@ -1089,3 +1115,5 @@ sub Marpa::R2::Recognizer::show_earley_sets {
 } ## end sub Marpa::R2::Recognizer::show_earley_sets
 
 1;
+
+# vim: set expandtab shiftwidth=4:
