@@ -1,4 +1,4 @@
-# Copyright 2013 Jeffrey Kegler
+# Copyright 2015 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -20,7 +20,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '2.073_001';
+$VERSION        = '2.103_008';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -65,6 +65,7 @@ action
 action_name
 adverb_item
 adverb_list
+adverb_list_items
 alternative
 alternatives
 array_descriptor
@@ -99,28 +100,28 @@ standard_name
 start_rule
 statement
 statements
-statement_body
 symbol
 symbol_name
 );
 
 
 $node_status{'Marpa::R2::Internal::MetaAST'} = q{};
+$node_status{array_descriptor} = "Actions in the form of array descriptors are not allowed";
 $node_status{character_class} = "Character classes are not allowed";
+$node_status{completion_event_declaration} = "Completion events are not allowed";
 $node_status{discard_rule} = ":discard rules are not allowed";
-$node_status{single_quoted_string} = "Quoted strings are not allowed";
+$node_status{event_specification} = qq{The "event" adverb is not allowed};
+$node_status{latm_specification} = qq{The "latm" adverb is not allowed};
 $node_status{lexeme_default_statement} = "The lexeme default statement is not allowed";
 $node_status{lexeme_rule} = "Lexeme statements are not allowed";
-$node_status{completion_event_declaration} = "Completion events are not allowed";
 $node_status{nulled_event_declaration} = "Nulled events are not allowed";
-$node_status{prediction_event_declaration} = "Prediction events are not allowed";
-$node_status{array_descriptor} = "Actions in the form of array descriptors are not allowed";
-$node_status{op_declare_match} = "G0 rules are not allowed";
-$node_status{priority_specification} = "The priority adverb is not allowed";
+$node_status{op_declare_match} = "lexical rules are not allowed";
 $node_status{pause_specification} = "The pause adverb is not allowed";
-# 'forgiving' was never documented and may be eliminated
-# $node_status{forgiving_specification} = qq{The "forgiving" adverb is not allowed};
-$node_status{event_specification} = qq{The "event" adverb is not allowed};
+$node_status{prediction_event_declaration} = "Prediction events are not allowed";
+$node_status{priority_specification} = "The priority adverb is not allowed";
+$node_status{single_quoted_string} = "Quoted strings are not allowed";
+$node_status{alternative_name} = "Alternative naming is not allowed";
+$node_status{naming} = "Alternative naming is not allowed";
 
 my %catch_error_node = 
     map { ; ($_ , 1 ) }
@@ -176,7 +177,8 @@ sub parse_rules {
     my $ast              = Marpa::R2::Internal::MetaAST->new($p_rules_source);
     {
         local $Marpa::R2::Internal::P_SOURCE = $p_rules_source;
-        my $problem = Marpa::R2::Internal::Stuifzand::check_ast_node($ast->{top_node});
+        my $problem = Marpa::R2::Internal::Stuifzand::check_ast_node(
+            $ast->{top_node} );
         ## Uncaught problem -- should not happen
         if ($problem) {
             Marpa::R2::exception(
@@ -186,6 +188,24 @@ sub parse_rules {
         } ## end if ($problem)
     }
     my $hashed_ast = $ast->ast_to_hash();
+    my $start_lhs = $hashed_ast->{'start_lhs'} // $hashed_ast->{'first_lhs'};
+    Marpa::R2::exception( 'No rules in Stuifzand grammar', )
+        if not defined $start_lhs;
+
+    my $internal_start_lhs = '[:start]';
+    $hashed_ast->{'default_g1_start_action'} =
+        $hashed_ast->{'default_adverbs'}->{'G1'}->{'action'};
+    $hashed_ast->{'symbols'}->{'G1'}->{$internal_start_lhs} = {
+        display_form => ':start',
+        description  => 'Internal G1 start symbol'
+    };
+    push @{ $hashed_ast->{rules}->{G1} },
+        {
+        lhs    => $internal_start_lhs,
+        rhs    => [$start_lhs],
+        action => '::first'
+        };
+
     $self->{rules} = $hashed_ast->{rules}->{G1};
     return $self;
 } ## end sub parse_rules
